import re
import os
from pathlib import Path

import gdgen
from gdgen import common
from gdgen import methods
from gdgen import gdtypes


class TemplateWriter:
	src = ''
	dest = ''
	
	def __init__(self, src, dest):
		self.src = src
		self.dest = dest
		
	def write_out(self, template={}):
		with open(self.src, 'r') as src_template:
			text = src_template.read()

		for placeholder, value in template.items():
			text = text.replace(placeholder, value)
		
		with open(self.dest, 'w') as dest_file:
			dest_file.write(text)
	

class FileWriter:
	dest = ''
	
	def __init__(self, dest, enc="utf-8"):
		self.dest = dest
		self.f = open(dest, "w", encoding=enc)
		
	def write_line(self, line="", ident_count=0):
		self.f.write("\t" * ident_count + line + "\n")
		
	def close(self):
		self.f.close()


def configure(module):
	gdtypes.update(module)


def make_config(module):
	config_dest = os.path.join(module.path, "config.py")
	config = FileWriter(config_dest)
	
	config.write_line()
	
	ver = module.get_engine_version(False)
	if ver['major'] >= 3:
		# 3.0 vs 3.1: https://github.com/godotengine/godot/pull/19275
		if ver['minor'] >= 1:
			config.write_line("def can_build(env, platform):")
		else:
			config.write_line("def can_build(env):")
	
		config.write_line("return True", 1)
		config.write_line()

	config.write_line("def configure(env):")
	config.write_line("pass", 1)
	config.write_line()
	
	if module.get_docs_path():
		config.write_line("def get_doc_path():")
		config.write_line("return \"" + module.get_docs_path() + "\"", 1)
		config.write_line()
		
		config.write_line("def get_doc_classes():")
		config.write_line("return [", 1)
		
		for c in module.get_classes():
			name = c['name']
			if not name:
				name = module.get_default_class_name()
			config.write_line('\"' + name + '\"' + ',', 2)
			
		config.write_line("]", 1)
		config.write_line()
		
	if module.get_icons_path():
		config.write_line("def get_icons_path():")
		config.write_line("return \"" + module.get_icons_path() + "\"", 1)
		config.write_line()
		
	config.close()

	
def make_readme(module):
	readme_dest = os.path.join(module.path, "README.md")
	readme = FileWriter(readme_dest)
	
	readme.write_line("#" + " " + module.get_name())
	readme.write_line()
	
	ver = module.get_engine_version()
	if ver == common.engine_latest_version:
		ver = "latest"
	
	readme.write_line("## Installation")
	readme.write_line()
	readme.write_line("Before installing, you must be able to")
	readme.write_line("[compile Godot Engine](https://docs.godotengine.org/en/" + ver + "/development/compiling/)")
	readme.write_line("from source.")
	readme.write_line()
	
	readme.write_line("```bash")
	readme.write_line("# Copy the module under directory named " + module.get_internal_name() + " (must be exactly that)")
	readme.write_line("cp " + module.get_internal_name() + " <godot_path>/modules/" + module.get_internal_name() + " && cd <godot_path>")
	readme.write_line("# Compile the engine manually, for instance:")
	readme.write_line("scons platform=linux target=release_debug bits=64")
	readme.write_line("```")
	
	readme.close()
	
	
def make_license(module, author):
	license_src = os.path.join(gdgen.get_path(), common.licenses_path, module.get_license()) + ".txt"
	license_dest = os.path.join(module.path, "LICENSE.txt")
	
	import datetime
	license_template = {
		"__YEAR__" : str(datetime.datetime.now().year),
		"__AUTHOR__" : author,
	}
	license_text = TemplateWriter(license_src, license_dest)
	license_text.write_out(license_template)
	
	
def make_register_types(module):
	
	# Header
	reg_types_header_dest = os.path.join(module.path, "register_types.h")
	header = FileWriter(reg_types_header_dest)
	
	header.write_line("void register_" + module.get_internal_name() + "_types();")
	header.write_line("void unregister_" + module.get_internal_name() + "_types();")
	header.close()
	
	# Source
	reg_types_source_dest = os.path.join(module.path, "register_types.cpp")
	source = FileWriter(reg_types_source_dest)
	
	source.write_line("#include \"register_types.h\"")
	source.write_line()
	
	for c in module.get_classes():
		name = methods.to_snake_case(c['name'])
		if not name:
			name = module.get_default_class_underscore_name()
		source.write_line("#include " + '\"' + name + ".h" + '\"')
		
	source.write_line()
	
	source.write_line("void register_" + module.get_internal_name() + "_types() {")
	source.write_line()
	
	for c in module.get_classes():
		name = c['name']
		if not name:
			name = module.get_default_class_name()
		source.write_line("ClassDB::register_class<" + name + ">();", 1)
		
	source.write_line("}")
	source.write_line()
	
	source.write_line("void unregister_" + module.get_internal_name() + "_types() {")
	source.write_line()
	source.write_line("// nothing to do here", 1)
	source.write_line("}")
	
	source.close()


def make_scsub(module):
	scsub_dest = os.path.join(module.path, "SCsub")
	scsub = FileWriter(scsub_dest)
	
	scsub.write_line("#!/usr/bin/env python")
	scsub.write_line()
	scsub.write_line("Import('env')")
	scsub.write_line("Import('env_modules')")
	scsub.write_line()

	env_module = "env_" + module.get_internal_name()
	
	scsub.write_line(env_module + " = env_modules.Clone()")
	scsub.write_line()
	
	if module.get_thirdparty_path():
		scsub.write_line("# Thirdparty source files")
		scsub.write_line("thirdparty_dir = '" + module.get_thirdparty_path() + "/'")
		scsub.write_line("thirdparty_sources = []")
		scsub.write_line("thirdparty_sources += Glob(thirdparty_dir + '**/*.cpp')")
		scsub.write_line("thirdparty_sources += Glob(thirdparty_dir + '**/*.c')")
		scsub.write_line()

		scsub.write_line(env_module + ".Prepend(CPPPATH=[thirdparty_dir])")
		scsub.write_line()
		
		scsub.write_line("env_thirdparty = " + env_module + ".Clone()")
		scsub.write_line("env_thirdparty.add_source_files(env.modules_sources, thirdparty_sources)")
		scsub.write_line("env_thirdparty.disable_warnings()")
		scsub.write_line()
		
	ver = module.get_engine_version(False)
	cpp_ver = module.get_cpp_version()
	
	if ver['major'] >= 3:
		if ver['minor'] >= 2 and cpp_ver == "c++11":
			pass # 3.2+ enables C++11 on the whole codebase by default
			# https://github.com/godotengine/godot/commit/5dae2ea777da5395cf1b1e9a8bc6abc93f6ae6bb
		else:
			scsub.write_line("if (not env.msvc):")
			scsub.write_line(env_module + ".Prepend(CXXFLAGS=['-std=" + cpp_ver + "'])", 1)
			scsub.write_line()
	
	scsub.write_line("# Module source files")
	
	scsub.write_line("source_dirs = [")
	
	for path in module.get_source_dirs():
		if not path:
			continue
		scsub.write_line("\"" + path + "/" + "\"" + ",", 1)
		
	scsub.write_line("]")
	
	scsub.write_line(env_module + ".Prepend(CPPPATH=source_dirs)")
	scsub.write_line("sources = [Glob(d + \"*.cpp\") for d in source_dirs]")
	scsub.write_line()
		
	scsub.write_line(env_module + ".add_source_files(env.modules_sources, sources)")
	
	scsub.close()
	
	
def make_classes(module):
	
	already_got_default = False
	
	for c_data in module.get_classes():
		name = c_data['name']
		
		if not name: # get default
			if already_got_default:
				continue
			class_name = module.get_default_class_name()
			underscore_name = module.get_internal_name()
			
			already_got_default = True
		else:
			class_name = name
			underscore_name = methods.to_snake_case(class_name)
			
		inherits = c_data['inherits']
		class_dir = os.path.join(module.path, c_data['path'])
		
		if not os.path.exists(class_dir):
			os.makedirs(class_dir)
			
		# Header
		header_dest = os.path.join(class_dir, underscore_name + '.h')
		write_class_header(header_dest, class_name, underscore_name, inherits)
		
		# Source
		source_dest = os.path.join(class_dir, underscore_name + '.cpp')
		write_class_source(source_dest, class_name, underscore_name, inherits)
		
		
def write_class_header(header_dest, name, underscore_name, inherits):
	
	header = FileWriter(header_dest)
	
	HEADER_GUARD = underscore_name.upper()
	
	header.write_line("#ifndef " + HEADER_GUARD + "_H")
	header.write_line("#define " + HEADER_GUARD + "_H")
	header.write_line()
	
	header.write_line(gdtypes.get_include(inherits))
	header.write_line()
	
	header.write_line("class " + name + " : " + "public " + inherits + " {")
	header.write_line("GDCLASS(" + name + ", " + inherits + ");", 1)
	header.write_line()
	
	header.write_line("protected:")
	header.write_line("static void _bind_methods();", 1)
	
	header.write_line("};")
	header.write_line()
	
	header.write_line("#endif " + "// " + HEADER_GUARD + "_H")
	
	header.close()
	
	
def write_class_source(source_dest, name, underscore_name, inherits):
	
	source = FileWriter(source_dest)
	
	source.write_line("#include " + '\"' + underscore_name + ".h" + '\"')
	source.write_line()
	
	source.write_line("void " + name + "::_bind_methods() {")
	source.write_line()
	source.write_line("}")
	
	source.close()
	
	
def make_gdignore(module):
	gdignore_dest = os.path.join(module.path, ".gdignore")
	Path(gdignore_dest).touch()



import os

import gdgen

from gdgen import common
from gdgen import methods
from gdgen import builders

from gdgen.vcs import get_providers as get_vcs_providers


class Module:
    def __init__(self, config, path):
        self.config = config
        self.path = path
        
    # Generate methods
    def generate(self):
	
        # Initialize directory structure
        os.makedirs(self.path)
        
        if self.get_docs_path():
            docs_dir = os.path.join(self.path, self.get_docs_path())
            os.makedirs(docs_dir)
            
        if self.get_icons_path():
            icons_dir = os.path.join(self.path, self.get_icons_path())
            os.makedirs(icons_dir)
        
        if self.get_thirdparty_path():
            thirdparty_dir = os.path.join(self.path, self.get_thirdparty_path())
            os.makedirs(thirdparty_dir)

        ### Generate!
        builders.configure(self)
        
        # Essential
        builders.make_config(self)
        builders.make_register_types(self)
        builders.make_scsub(self)
        
        builders.make_classes(self)
        
        # Optional
        if self.should_initialize_readme():
            builders.make_readme(self)
            
        if self.get_license():
            author = self.get_author()
            builders.make_license(self, author)
            
        if self.to_be_included_inside_project():
            builders.make_gdignore(self)
        
        if self.get_vcs():
            try:
                self.initialize_repository()
            except:
                pass
            
    
    def initialize_repository(self):
        
        for vcs in get_vcs_providers():
            if not vcs.can_handle(self.get_vcs()):
                continue
            vcs.initialize(self.path)

    # Config methods
        
    def get_name(self):
        return self.config.get('name', '')
        
    def get_internal_name(self):
        return self.config.get('internal_name', '').lower()
        
    def get_author(self):
        author = self.config.get('author', '')
            
        if not author:
            author = self.get_default_author()
                
        return author
        
    @staticmethod
    def get_default_author():
        author = ''
        try:
            for vcs in get_vcs_providers():
                author = vcs.get_author()
                if author:
                    break
        except:
            pass
        
        return author
        
    def get_engine_version(self, as_string=True):
        
        ver_str = self.config.get('engine_version', 'latest')
        
        if not as_string:
            if ver_str == 'latest':
                ver_str = common.engine_latest_version
                
            ver_comp = ver_str.split('.')
            ver = {
                'major': int(ver_comp[0]),
                'minor': int(ver_comp[1]),
            }
            return ver
            
        return ver_str
        
    @staticmethod
    def get_default_engine_version():
        return "latest"
    
    def get_cpp_version(self):
        return self.config.get('cpp_version', 'c++11').lower()
        
    @staticmethod
    def get_default_cpp_version():
        return 'c++11'
        
    def get_classes(self):
        return self.config.get('classes', [])
        
    def get_docs_path(self):
        return self.config.get('docs_path', '')
        
    @staticmethod
    def get_default_docs_path():
        return 'docs'
        
    def get_icons_path(self):
        return self.config.get('icons_path', '')
        
    @staticmethod
    def get_default_icons_path():
        return 'editor/icons'
        
    def get_thirdparty_path(self):
        return self.config.get('thirdparty_path', '')
        
    @staticmethod
    def get_default_thirdparty_path():
        return 'thirdparty'
        
    def should_initialize_readme(self):
        return self.config.get('readme', '')
        
    def get_license(self):
        return self.config.get('license', '')
        
    @staticmethod
    def get_default_license():
        return "MIT"
        
    def get_vcs(self):
        return self.config.get('version_control', '')
        
    @staticmethod
    def get_default_vcs():
        return "git"
        
    def get_changelog(self):
        return self.config.get('changelog', '')
        
    def get_ci(self):
        return self.config.get('continuous_integration', '')
        
    def to_be_included_inside_project(self):
        return self.config.get('to_be_included_inside_project', '')
    
    # Config utility methods
    
    def get_default_class_name(self):
        return methods.to_pascal_case(self.get_internal_name())
        
    def get_default_class_underscore_name(self):
	    return methods.to_snake_case(self.get_internal_name())
        
    def get_source_dirs(self):
        dirs = ['.']
        
        for c in self.get_classes():
            dirs.append(c['path'])
            
        return dirs



import re

def to_snake_case(camel):
	# https://stackoverflow.com/a/33516645/
    return re.sub(r'([A-Z]*)([A-Z][a-z]+)', lambda x: (x.group(1) + '_' if x.group(1) else '') + x.group(2) + '_', camel).rstrip('_').lower()
	
	
def to_pascal_case(snake):
	# https://stackoverflow.com/a/19053800/
    comp = snake.split('_')
    return comp[0].title() + ''.join(x.title() for x in comp[1:])



from unittest import TestCase
import os

import gdgen
from gdgen import gdmodule


class GDModuleTest(TestCase):
    
    modules_dir = os.path.join(gdgen.get_path(), "tests/modules/")
    
    sample_config = "tests/configs/sample.json"

    @classmethod
    def setUpClass(cls):
        import shutil
        shutil.rmtree(cls.modules_dir, True)
        os.makedirs(cls.modules_dir)
    
    
    def test_init_from_default_config(self):
        config = gdmodule.load_config(gdmodule.get_default_config_path())
        config['name'] = "Default Module 3D"
        config['internal_name'] = "default_module_3d"
        gdmodule.init(config, self.modules_dir)
        
        self.current_dir = "default_module_3d/"
        
        self.should_exist('docs/')
        self.should_exist('editor/icons/')
        self.should_exist('thirdparty/')
        self.should_exist('.gitattributes')
        self.should_exist('.gitignore')
        self.should_exist('config.py')
        self.should_exist('default_module_3d.cpp')
        self.should_exist('default_module_3d.h')
        self.should_exist('LICENSE.txt')
        self.should_exist('README.md')
        self.should_exist('register_types.cpp')
        self.should_exist('register_types.h')
        self.should_exist('SCsub')
        
        self.should_not_exist('.gdignore')
        self.should_not_exist('classes/')
        
    
    def test_init_from_minimal_config(self):
        config = gdmodule.load_config(gdmodule.get_minimal_config_path())
        config['name'] = "Minimal Module"
        config['internal_name'] = "MINIMAL_MODULE"
        gdmodule.init(config, self.modules_dir)
    
        self.current_dir = "minimal_module/"
        
        self.assertTrue('minimal_module' in os.listdir(self.modules_dir), "Case mismatch detected!")
        
        self.should_exist('config.py')
        self.should_exist('register_types.cpp')
        self.should_exist('register_types.h')
        self.should_exist('SCsub')
        
        self.should_not_exist('docs/')
        self.should_not_exist('editor/icons/')
        self.should_not_exist('thirdparty/')
        self.should_not_exist('.gitattributes')
        self.should_not_exist('.gitignore')
        self.should_not_exist('.gdignore')
        self.should_not_exist('classes/')
        self.should_not_exist('minimal_module.cpp')
        self.should_not_exist('minimal_module.h')
        self.should_not_exist('LICENSE.txt')
        self.should_not_exist('README.md')
    
    
    def test_init_from_sample_config(self):
        config = gdmodule.load_config(self.sample_config)
        config['name'] = "Sample Module 2D"
        config['internal_name'] = "sample_module_2D"
        gdmodule.init(config, self.modules_dir)
        
        self.current_dir = "sample_module_2d/"
        
        self.assertTrue('sample_module_2d' in os.listdir(self.modules_dir), "Case mismatch detected!")
        
        self.should_exist('.git')
        
        self.should_exist('classes_a/')
        self.should_exist('classes_a/new_node_2d.cpp')
        self.should_exist('classes_a/new_node_2d.h')
        
        self.should_exist('classes_b/')
        self.should_exist('classes_b/new_reference.cpp')
        self.should_exist('classes_b/new_reference.h')
        
        self.should_exist('docs/')
        self.should_exist('editor/icons/')
        self.should_exist('thirdparty/')
        self.should_exist('.gdignore')
        self.should_exist('.gitattributes')
        self.should_exist('.gitignore')
        self.should_exist('config.py')
        self.should_exist('LICENSE.txt')
        self.should_exist('README.md')
        self.should_exist('register_types.cpp')
        self.should_exist('register_types.h')
        self.should_exist('sample_module_2d.cpp')
        self.should_exist('sample_module_2d.h')
        self.should_exist('SCsub')
        
        
    def exists(self, path):
        return os.path.exists(self.modules_dir + self.current_dir + path)
        
        
    def should_exist(self, path):
        return self.assertTrue(self.exists(path))
        
        
    def should_not_exist(self, path):
        return self.assertFalse(self.exists(path))
        

if __name__ == '__main__':
    from unittest import main
    main()



#!/usr/bin/env python3

import os
import json

import gdgen

from gdgen import common
from gdgen import methods
from gdgen.messages import *
from gdgen.module import Module


def init(config, output_path="", force=False):
		
	# Ensure Godot's convention is followed for module naming
	config['internal_name'] = methods.to_snake_case(config['internal_name'])
		
	if output_path:
		module_path = os.path.join(output_path, config['internal_name'])
	else:
		module_path = os.path.join(os.getcwd(), config['internal_name'])
		
	# Generate module
	m = Module(config, module_path)
	try:
		if os.path.exists(module_path) and force:
			deinit(module_path)
		m.generate()
	
	except Exception as e:
		print_info(str(e))
		if not force:
			print_info(INFO_PASS_FORCE_CMD)
		return
		

def load_config(path):
	config = {}
	try:
		if not os.path.isabs(path):
			config_path = os.path.join(gdgen.get_path(), path)
		
		with open(config_path) as config_data:
			config = json.load(config_data)
			
	except Exception as e:
		print_info('config: ' + str(e))
		
	return config


def get_default_config_path():
	return common.config_default_path
	
	
def get_minimal_config_path():
	return common.config_minimal_path

		
def deinit(path):
	import shutil
	shutil.rmtree(path)


def print_info(msg):
	print(__file__ + ': ' + msg)



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="gdgen",
    version="1.0a",
    author="Andrii Doroshenko (Xrayez)",
    author_email="xrayez@gmail.com",
    description="Code generator for Godot Engine",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/Xrayez/gdgen",
    packages=setuptools.find_packages(),
    entry_points = {
        "console_scripts": [
            "gdgen = gdgen.cli:main"
        ]
    },
    include_package_data=True,
    license="MIT",
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Programming Language :: Python :: 3",
        "Intended Audience :: Developers",
        "Topic :: Software Development :: Code Generators",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    setup_requires=["wheel"],
    python_requires='>=3.6',
)



#!/usr/bin/env python3

import os
import argparse

import gdgen

from gdgen import common
from gdgen import gdmodule
from gdgen.module import Module


def main():
	# Parse command line arguments
	parser = argparse.ArgumentParser()

	parser.add_argument('-n', '--name', default="")
	parser.add_argument('-s', '--internal-name', default="")

	parser.add_argument('-c', '--config-path', default=common.config_default_path)
	parser.add_argument('-o', '--output-path', default="")

	parser.add_argument('-f', '--force', action="store_true")

	args = parser.parse_args()
	
	# Load config
	config = gdmodule.load_config(args.config_path)
	
	if args.name and args.internal_name:
		# Override
		config['name'] = args.name
		config['internal_name'] = args.internal_name
	else:
		try:
			# Run wizard
			while config.get('name') == None or config.get('name') == "":
				config['name'] = read_input("Module name")
				
			while config.get('internal_name') == None or config.get('internal_name') == "":
				config['internal_name'] = read_input("Internal module name (snake_case)")
			
			config['author'] = read_input("Author name", Module.get_default_author())
			config['engine_version'] = read_input("Engine version", Module.get_default_engine_version())
			config['cpp_version'] = read_input("C++ version", Module.get_default_cpp_version())
			
			num_classes = read_input("Number of classes to generate", 0)
			if not num_classes:
				num_classes = 0
			
			classes = []
			for n in range(int(num_classes)):
				class_name = read_input("(%s) Class name" % n)
				class_inherits = read_input("(%s) Class inherits" % n)
				class_path = read_input("(%s) Class path" % n, "")
				
				classes.append( {
						"name": class_name,
						"inherits": class_inherits,
						"path": class_path
					}
				)
			config['classes'] = classes
			
			config['docs_path'] = read_input("Documentation path", Module.get_default_docs_path())
			config['icons_path'] = read_input("Icons path", Module.get_default_icons_path())
			config['thirdparty_path'] = read_input("Thirdparty path", Module.get_default_thirdparty_path())
			
			config['readme'] = read_input("Initialize README?", True)
			config['license'] = read_input("License", Module.get_default_license())
			config['version_control'] = read_input("Version control system", Module.get_default_vcs())
			config['to_be_included_inside_project'] = read_input("Will be included inside project?", False)
			
		except KeyboardInterrupt:
			print_info("\nSkipping the wizard.")
	
	# Validate config
	try:
		validate_config(config)
	except ValueError as e:
		print_info(str(e).lower())
		return
	
	# Generate
	gdmodule.init(config, args.output_path, args.force)
	
	
def read_input(prompt, default_value=None):
	if default_value != None:
		msg = "%s (default - %s): " % (prompt, str(default_value))
	else:
		msg = "%s: " % (prompt)
		
	value = input(msg)
	if not value:
		value = default_value
		
	return value
	
	
def validate_config(config):
	required = [
		'name', 
		'internal_name'
	]
	validated = []
	
	for param in config:
		if param in required:
			validated.append(param)
	
	if len(validated) != len(required):
		missing = set(required) - set(validated)
		raise ValueError("Missing required fields in config: %s" % missing)
	
	for param in required:
		if not config[param]:
			raise ValueError("Required config parameter not set: %s" % param)


def print_info(msg):
	print(__file__ + ': ' + msg)



from gdgen import methods

includes = {}
types = {}


def update(module):
	# In that order
	update_types(module)
	make_includes(module)


def update_types(module):
	global types
	types.clear()
	
	ver = module.get_engine_version(False)
	
	basedir = ""
	
	if ver['major'] >= 3:
		if ver['minor'] >= 1:
			# https://github.com/godotengine/godot/pull/21978
			basedir = "core/"
			
		types['Object'] = basedir
		types['Reference'] = basedir
		types['Resource'] = basedir
		
		basedir = "scene/main/"
		types['Node'] = basedir
		
		basedir = "scene/2d/"
		types['Node2D'] = basedir


def make_includes(module):
	global includes 
	includes.clear()
	
	include_str = "#include \"%s%s.h\""
	
	for t, basedir in types.items():
		class_type = methods.to_snake_case(t)
		includes[t] = include_str % (basedir, class_type)


def get_include(class_type):
	if class_type in includes:
		return includes[class_type]
	else: # fallback
		include_str = "#include \"%s.h\""
		return include_str % methods.to_snake_case(class_type)



import subprocess
import os

import gdgen

from gdgen import common
from gdgen.builders import TemplateWriter


class VCSProvider:
    
    @classmethod
    def initialize(cls, path):
        pass
    
    @staticmethod
    def get_name():
        return ''
    
    @classmethod
    def get_templates_path(cls):
        return ''
        
    @classmethod
    def can_handle(cls, name):
        return cls.get_name() == name
        
    @staticmethod
    def get_author():
        return ""
    
    
class VCSGit(VCSProvider):
    
    @classmethod
    def initialize(cls, path):
        subprocess.run(['git', 'init'], cwd=path).check_returncode()
        
        templates = os.listdir(cls.get_templates_path())
        
        for f in templates:
            src = os.path.join(cls.get_templates_path(), f)
            dest = os.path.join(path, f)
            
            tw = TemplateWriter(src, dest)
            tw.write_out()
        
    @staticmethod
    def get_name():
        return 'git'
        
    @classmethod
    def get_templates_path(cls):
        return os.path.join(gdgen.get_path(), common.vcs_path, cls.get_name())
        
    @staticmethod
    def get_author():
        git_author = ['git', 'config', 'user.name']
        output = subprocess.check_output(git_author, shell=True)
        author = output.decode("utf-8").strip("\n")
        return author
        
    
def get_providers():
    return [VCSGit]
